@@ -1,13 +1,3 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.test
-    ~~~~~~~~~~
-
-    Tests the testing tools.
-
-    :copyright: 2007 Pallets
-    :license: BSD-3-Clause
-"""
 import json
 import sys
 from functools import partial
@@ -15,13 +5,13 @@ from io import BytesIO
 
 import pytest
 
-from . import strict_eq
-from werkzeug._compat import implements_iterator
-from werkzeug._compat import iteritems
-from werkzeug._compat import to_bytes
+from werkzeug._internal import _to_bytes
+from werkzeug.datastructures import Authorization
 from werkzeug.datastructures import FileStorage
+from werkzeug.datastructures import Headers
 from werkzeug.datastructures import MultiDict
 from werkzeug.formparser import parse_form_data
+from werkzeug.http import parse_authorization_header
 from werkzeug.test import Client
 from werkzeug.test import ClientRedirectError
 from werkzeug.test import create_environ
@@ -29,7 +19,6 @@ from werkzeug.test import EnvironBuilder
 from werkzeug.test import run_wsgi_app
 from werkzeug.test import stream_encode_multipart
 from werkzeug.utils import redirect
-from werkzeug.wrappers import BaseResponse
 from werkzeug.wrappers import Request
 from werkzeug.wrappers import Response
 from werkzeug.wsgi import pop_path_info
@@ -56,11 +45,11 @@ def redirect_with_get_app(environ, start_response):
         "http://localhost/first/request",
         "http://localhost/some/redirect/",
     ):
-        assert False, 'redirect_demo_app() did not expect URL "%s"' % req.url
+        raise AssertionError(f'redirect_demo_app() did not expect URL "{req.url}"')
     if "/some/redirect" not in req.url:
         response = redirect("http://localhost/some/redirect/")
     else:
-        response = Response("current url: %s" % req.url)
+        response = Response(f"current url: {req.url}")
     return response(environ, start_response)
 
 
@@ -88,47 +77,47 @@ def multi_value_post_app(environ, start_response):
 def test_cookie_forging():
     c = Client(cookie_app)
     c.set_cookie("localhost", "foo", "bar")
-    appiter, code, headers = c.open()
-    strict_eq(list(appiter), [b"foo=bar"])
+    response = c.open()
+    assert response.data == b"foo=bar"
 
 
 def test_set_cookie_app():
     c = Client(cookie_app)
-    appiter, code, headers = c.open()
-    assert "Set-Cookie" in dict(headers)
+    response = c.open()
+    assert "Set-Cookie" in response.headers
 
 
 def test_cookiejar_stores_cookie():
     c = Client(cookie_app)
-    appiter, code, headers = c.open()
+    c.open()
     assert "test" in c.cookie_jar._cookies["localhost.local"]["/"]
 
 
 def test_no_initial_cookie():
     c = Client(cookie_app)
-    appiter, code, headers = c.open()
-    strict_eq(b"".join(appiter), b"No Cookie")
+    response = c.open()
+    assert response.data == b"No Cookie"
 
 
 def test_resent_cookie():
     c = Client(cookie_app)
     c.open()
-    appiter, code, headers = c.open()
-    strict_eq(b"".join(appiter), b"test=test")
+    response = c.open()
+    assert response.data == b"test=test"
 
 
 def test_disable_cookies():
     c = Client(cookie_app, use_cookies=False)
     c.open()
-    appiter, code, headers = c.open()
-    strict_eq(b"".join(appiter), b"No Cookie")
+    response = c.open()
+    assert response.data == b"No Cookie"
 
 
 def test_cookie_for_different_path():
     c = Client(cookie_app)
     c.open("/path1")
-    appiter, code, headers = c.open("/path2")
-    strict_eq(b"".join(appiter), b"test=test")
+    response = c.open("/path2")
+    assert response.data == b"test=test"
 
 
 def test_environ_builder_basics():
@@ -146,12 +135,12 @@ def test_environ_builder_basics():
     req = b.get_request()
     b.close()
 
-    strict_eq(req.url, u"http://localhost/")
-    strict_eq(req.method, "POST")
-    strict_eq(req.form["test"], u"normal value")
+    assert req.url == "http://localhost/"
+    assert req.method == "POST"
+    assert req.form["test"] == "normal value"
     assert req.files["test"].content_type == "text/plain"
-    strict_eq(req.files["test"].filename, u"test.txt")
-    strict_eq(req.files["test"].read(), b"test contents")
+    assert req.files["test"].filename == "test.txt"
+    assert req.files["test"].read() == b"test contents"
 
 
 def test_environ_builder_data():
@@ -188,7 +177,7 @@ def test_environ_builder_json():
         assert request.content_type == "application/json"
         return Response(json.loads(request.get_data(as_text=True))["foo"])
 
-    c = Client(app, Response)
+    c = Client(app)
     response = c.post("/", json={"foo": "bar"})
     assert response.data == b"bar"
 
@@ -203,13 +192,13 @@ def test_environ_builder_headers():
     )
     b.headers["X-Beat-My-Horse"] = "very well sir"
     env = b.get_environ()
-    strict_eq(env["HTTP_USER_AGENT"], "Foo/0.1")
-    strict_eq(env["HTTP_X_BEAT_MY_HORSE"], "very well sir")
-    strict_eq(env["wsgi.version"], (1, 1))
+    assert env["HTTP_USER_AGENT"] == "Foo/0.1"
+    assert env["HTTP_X_BEAT_MY_HORSE"] == "very well sir"
+    assert env["wsgi.version"] == (1, 1)
 
     b.headers["User-Agent"] = "Bar/1.0"
     env = b.get_environ()
-    strict_eq(env["HTTP_USER_AGENT"], "Bar/1.0")
+    assert env["HTTP_USER_AGENT"] == "Bar/1.0"
 
 
 def test_environ_builder_headers_content_type():
@@ -224,37 +213,46 @@ def test_environ_builder_headers_content_type():
     assert "CONTENT_TYPE" not in env
 
 
+def test_envrion_builder_multiple_headers():
+    h = Headers()
+    h.add("FOO", "bar")
+    h.add("FOO", "baz")
+    b = EnvironBuilder(headers=h)
+    env = b.get_environ()
+    assert env["HTTP_FOO"] == "bar, baz"
+
+
 def test_environ_builder_paths():
     b = EnvironBuilder(path="/foo", base_url="http://example.com/")
-    strict_eq(b.base_url, "http://example.com/")
-    strict_eq(b.path, "/foo")
-    strict_eq(b.script_root, "")
-    strict_eq(b.host, "example.com")
+    assert b.base_url == "http://example.com/"
+    assert b.path == "/foo"
+    assert b.script_root == ""
+    assert b.host == "example.com"
 
     b = EnvironBuilder(path="/foo", base_url="http://example.com/bar")
-    strict_eq(b.base_url, "http://example.com/bar/")
-    strict_eq(b.path, "/foo")
-    strict_eq(b.script_root, "/bar")
-    strict_eq(b.host, "example.com")
+    assert b.base_url == "http://example.com/bar/"
+    assert b.path == "/foo"
+    assert b.script_root == "/bar"
+    assert b.host == "example.com"
 
     b.host = "localhost"
-    strict_eq(b.base_url, "http://localhost/bar/")
+    assert b.base_url == "http://localhost/bar/"
     b.base_url = "http://localhost:8080/"
-    strict_eq(b.host, "localhost:8080")
-    strict_eq(b.server_name, "localhost")
-    strict_eq(b.server_port, 8080)
+    assert b.host == "localhost:8080"
+    assert b.server_name == "localhost"
+    assert b.server_port == 8080
 
     b.host = "foo.invalid"
     b.url_scheme = "https"
     b.script_root = "/test"
     env = b.get_environ()
-    strict_eq(env["SERVER_NAME"], "foo.invalid")
-    strict_eq(env["SERVER_PORT"], "443")
-    strict_eq(env["SCRIPT_NAME"], "/test")
-    strict_eq(env["PATH_INFO"], "/foo")
-    strict_eq(env["HTTP_HOST"], "foo.invalid")
-    strict_eq(env["wsgi.url_scheme"], "https")
-    strict_eq(b.base_url, "https://foo.invalid/test/")
+    assert env["SERVER_NAME"] == "foo.invalid"
+    assert env["SERVER_PORT"] == "443"
+    assert env["SCRIPT_NAME"] == "/test"
+    assert env["PATH_INFO"] == "/foo"
+    assert env["HTTP_HOST"] == "foo.invalid"
+    assert env["wsgi.url_scheme"] == "https"
+    assert b.base_url == "https://foo.invalid/test/"
 
 
 def test_environ_builder_content_type():
@@ -275,12 +273,28 @@ def test_environ_builder_content_type():
     builder.files.add_file("blafasel", BytesIO(b"foo"), "test.txt")
     assert builder.content_type == "multipart/form-data"
     req = builder.get_request()
-    strict_eq(req.form["foo"], u"bar")
-    strict_eq(req.files["blafasel"].read(), b"foo")
+    assert req.form["foo"] == "bar"
+    assert req.files["blafasel"].read() == b"foo"
+
+
+def test_basic_auth():
+    builder = EnvironBuilder(auth=("username", "password"))
+    request = builder.get_request()
+    auth = parse_authorization_header(request.headers["Authorization"])
+    assert auth.username == "username"
+    assert auth.password == "password"
+
+
+def test_auth_object():
+    builder = EnvironBuilder(
+        auth=Authorization("digest", {"username": "u", "password": "p"})
+    )
+    request = builder.get_request()
+    assert request.headers["Authorization"].startswith("Digest ")
 
 
 def test_environ_builder_stream_switch():
-    d = MultiDict(dict(foo=u"bar", blub=u"blah", hu=u"hum"))
+    d = MultiDict(dict(foo="bar", blub="blah", hu="hum"))
     for use_tempfile in False, True:
         stream, length, boundary = stream_encode_multipart(
             d, use_tempfile, threshold=150
@@ -291,17 +305,17 @@ def test_environ_builder_stream_switch():
             {
                 "wsgi.input": stream,
                 "CONTENT_LENGTH": str(length),
-                "CONTENT_TYPE": 'multipart/form-data; boundary="%s"' % boundary,
+                "CONTENT_TYPE": f'multipart/form-data; boundary="{boundary}"',
             }
         )[1]
-        strict_eq(form, d)
+        assert form == d
         stream.close()
 
 
 def test_environ_builder_unicode_file_mix():
     for use_tempfile in False, True:
-        f = FileStorage(BytesIO(u"\N{SNOWMAN}".encode("utf-8")), "snowman.txt")
-        d = MultiDict(dict(f=f, s=u"\N{SNOWMAN}"))
+        f = FileStorage(BytesIO(rb"\N{SNOWMAN}"), "snowman.txt")
+        d = MultiDict(dict(f=f, s="\N{SNOWMAN}"))
         stream, length, boundary = stream_encode_multipart(
             d, use_tempfile, threshold=150
         )
@@ -311,13 +325,13 @@ def test_environ_builder_unicode_file_mix():
             {
                 "wsgi.input": stream,
                 "CONTENT_LENGTH": str(length),
-                "CONTENT_TYPE": 'multipart/form-data; boundary="%s"' % boundary,
+                "CONTENT_TYPE": f'multipart/form-data; boundary="{boundary}"',
             }
         )
-        strict_eq(form["s"], u"\N{SNOWMAN}")
-        strict_eq(files["f"].name, "f")
-        strict_eq(files["f"].filename, u"snowman.txt")
-        strict_eq(files["f"].read(), u"\N{SNOWMAN}".encode("utf-8"))
+        assert form["s"] == "\N{SNOWMAN}"
+        assert files["f"].name == "f"
+        assert files["f"].filename == "snowman.txt"
+        assert files["f"].read() == rb"\N{SNOWMAN}"
         stream.close()
 
 
@@ -339,10 +353,10 @@ def test_create_environ():
         "SERVER_PROTOCOL": "HTTP/1.1",
         "QUERY_STRING": "bar=baz",
     }
-    for key, value in iteritems(expected):
+    for key, value in iter(expected.items()):
         assert env[key] == value
-    strict_eq(env["wsgi.input"].read(0), b"")
-    strict_eq(create_environ("/foo", "http://example.com/")["SCRIPT_NAME"], "")
+    assert env["wsgi.input"].read(0) == b""
+    assert create_environ("/foo", "http://example.com/")["SCRIPT_NAME"] == ""
 
 
 def test_create_environ_query_string_error():
@@ -352,24 +366,26 @@ def test_create_environ_query_string_error():
 
 def test_builder_from_environ():
     environ = create_environ(
-        "/foo",
+        "/ㄱ",
         base_url="https://example.com/base",
         query_string={"name": "Werkzeug"},
-        data={"foo": "bar"},
-        headers={"X-Foo": "bar"},
+        data={"foo": "ㄴ"},
+        headers={"X-Foo": "ㄷ"},
     )
     builder = EnvironBuilder.from_environ(environ)
+
     try:
         new_environ = builder.get_environ()
     finally:
         builder.close()
+
     assert new_environ == environ
 
 
 def test_file_closing():
     closed = []
 
-    class SpecialInput(object):
+    class SpecialInput:
         def read(self, size):
             return ""
 
@@ -377,35 +393,35 @@ def test_file_closing():
             closed.append(self)
 
     create_environ(data={"foo": SpecialInput()})
-    strict_eq(len(closed), 1)
+    assert len(closed) == 1
     builder = EnvironBuilder()
     builder.files.add_file("blah", SpecialInput())
     builder.close()
-    strict_eq(len(closed), 2)
+    assert len(closed) == 2
 
 
 def test_follow_redirect():
     env = create_environ("/", base_url="http://localhost")
     c = Client(redirect_with_get_app)
-    appiter, code, headers = c.open(environ_overrides=env, follow_redirects=True)
-    strict_eq(code, "200 OK")
-    strict_eq(b"".join(appiter), b"current url: http://localhost/some/redirect/")
+    response = c.open(environ_overrides=env, follow_redirects=True)
+    assert response.status == "200 OK"
+    assert response.data == b"current url: http://localhost/some/redirect/"
 
     # Test that the :cls:`Client` is aware of user defined response wrappers
-    c = Client(redirect_with_get_app, response_wrapper=BaseResponse)
+    c = Client(redirect_with_get_app)
     resp = c.get("/", follow_redirects=True)
-    strict_eq(resp.status_code, 200)
-    strict_eq(resp.data, b"current url: http://localhost/some/redirect/")
+    assert resp.status_code == 200
+    assert resp.data == b"current url: http://localhost/some/redirect/"
 
     # test with URL other than '/' to make sure redirected URL's are correct
-    c = Client(redirect_with_get_app, response_wrapper=BaseResponse)
+    c = Client(redirect_with_get_app)
     resp = c.get("/first/request", follow_redirects=True)
-    strict_eq(resp.status_code, 200)
-    strict_eq(resp.data, b"current url: http://localhost/some/redirect/")
+    assert resp.status_code == 200
+    assert resp.data == b"current url: http://localhost/some/redirect/"
 
 
 def test_follow_local_redirect():
-    class LocalResponse(BaseResponse):
+    class LocalResponse(Response):
         autocorrect_location_header = False
 
     def local_redirect_app(environ, start_response):
@@ -413,13 +429,13 @@ def test_follow_local_redirect():
         if "/from/location" in req.url:
             response = redirect("/to/location", Response=LocalResponse)
         else:
-            response = Response("current path: %s" % req.path)
+            response = Response(f"current path: {req.path}")
         return response(environ, start_response)
 
-    c = Client(local_redirect_app, response_wrapper=BaseResponse)
+    c = Client(local_redirect_app)
     resp = c.get("/from/location", follow_redirects=True)
-    strict_eq(resp.status_code, 200)
-    strict_eq(resp.data, b"current path: /to/location")
+    assert resp.status_code == 200
+    assert resp.data == b"current path: /to/location"
 
 
 @pytest.mark.parametrize(
@@ -437,11 +453,11 @@ def test_follow_redirect_body(code, keep):
             else:
                 assert not request.form
 
-            return Response("current url: %s" % request.url)
+            return Response(f"current url: {request.url}")
 
         return redirect("http://localhost/some/redirect/", code=code)
 
-    c = Client(app, response_wrapper=BaseResponse)
+    c = Client(app)
     response = c.post(
         "/", follow_redirects=True, data={"foo": "bar"}, headers={"X-Foo": "bar"}
     )
@@ -476,7 +492,7 @@ def test_follow_external_redirect_on_same_subdomain():
 
 
 def test_follow_redirect_loop():
-    c = Client(redirect_loop_app, response_wrapper=BaseResponse)
+    c = Client(redirect_loop_app)
     with pytest.raises(ClientRedirectError):
         c.get("/", follow_redirects=True)
 
@@ -489,7 +505,7 @@ def test_follow_redirect_non_root_base_url():
 
         return Response(request.path)
 
-    c = Client(app, response_wrapper=Response)
+    c = Client(app)
     response = c.get(
         "/redirect", base_url="http://localhost/other", follow_redirects=True
     )
@@ -497,7 +513,7 @@ def test_follow_redirect_non_root_base_url():
 
 
 def test_follow_redirect_exhaust_intermediate():
-    class Middleware(object):
+    class Middleware:
         def __init__(self, app):
             self.app = app
             self.active = 0
@@ -509,18 +525,45 @@ def test_follow_redirect_exhaust_intermediate():
             assert not self.active
             self.active += 1
             try:
-                for chunk in self.app(environ, start_response):
-                    yield chunk
+                yield from self.app(environ, start_response)
             finally:
                 self.active -= 1
 
     app = Middleware(redirect_with_get_app)
-    client = Client(Middleware(redirect_with_get_app), Response)
+    client = Client(Middleware(redirect_with_get_app))
     response = client.get("/", follow_redirects=True, buffered=False)
     assert response.data == b"current url: http://localhost/some/redirect/"
     assert not app.active
 
 
+def test_redirects_are_tracked():
+    @Request.application
+    def app(request):
+        if request.path == "/first":
+            return redirect("/second")
+
+        if request.path == "/second":
+            return redirect("/third")
+
+        return Response("done")
+
+    c = Client(app)
+    response = c.get("/first", follow_redirects=True)
+    assert response.data == b"done"
+    assert len(response.history) == 2
+
+    assert response.history[-1].request.path == "/second"
+    assert response.history[-1].status_code == 302
+    assert response.history[-1].location == "http://localhost/third"
+    assert len(response.history[-1].history) == 1
+    assert response.history[-1].history[-1] is response.history[-2]
+
+    assert response.history[-2].request.path == "/first"
+    assert response.history[-2].status_code == 302
+    assert response.history[-2].location == "http://localhost/second"
+    assert len(response.history[-2].history) == 0
+
+
 def test_cookie_across_redirect():
     @Request.application
     def app(request):
@@ -537,7 +580,7 @@ def test_cookie_across_redirect():
             rv.delete_cookie("auth")
             return rv
 
-    c = Client(app, Response)
+    c = Client(app)
     assert c.get("/").data == b"out"
     assert c.get("/in", follow_redirects=True).data == b"in"
     assert c.get("/").data == b"in"
@@ -558,7 +601,7 @@ def test_redirect_mutate_environ():
         pop_path_info(environ)
         return app(environ, start_response)
 
-    c = Client(middleware, Response)
+    c = Client(middleware)
     rv = c.get("/prefix/first", follow_redirects=True)
     # if modified environ was used by client, this would be /
     assert rv.data == b"/second"
@@ -567,31 +610,31 @@ def test_redirect_mutate_environ():
 def test_path_info_script_name_unquoting():
     def test_app(environ, start_response):
         start_response("200 OK", [("Content-Type", "text/plain")])
-        return [environ["PATH_INFO"] + "\n" + environ["SCRIPT_NAME"]]
+        return [f"{environ['PATH_INFO']}\n{environ['SCRIPT_NAME']}"]
 
-    c = Client(test_app, response_wrapper=BaseResponse)
+    c = Client(test_app)
     resp = c.get("/foo%40bar")
-    strict_eq(resp.data, b"/foo@bar\n")
-    c = Client(test_app, response_wrapper=BaseResponse)
+    assert resp.data == b"/foo@bar\n"
+    c = Client(test_app)
     resp = c.get("/foo%40bar", "http://localhost/bar%40baz")
-    strict_eq(resp.data, b"/foo@bar\n/bar@baz")
+    assert resp.data == b"/foo@bar\n/bar@baz"
 
 
 def test_multi_value_submit():
-    c = Client(multi_value_post_app, response_wrapper=BaseResponse)
+    c = Client(multi_value_post_app)
     data = {"field": ["val1", "val2"]}
     resp = c.post("/", data=data)
-    strict_eq(resp.status_code, 200)
-    c = Client(multi_value_post_app, response_wrapper=BaseResponse)
+    assert resp.status_code == 200
+    c = Client(multi_value_post_app)
     data = MultiDict({"field": ["val1", "val2"]})
     resp = c.post("/", data=data)
-    strict_eq(resp.status_code, 200)
+    assert resp.status_code == 200
 
 
 def test_iri_support():
-    b = EnvironBuilder(u"/föö-bar", base_url=u"http://☃.net/")
-    strict_eq(b.path, "/f%C3%B6%C3%B6-bar")
-    strict_eq(b.base_url, "http://xn--n3h.net/")
+    b = EnvironBuilder("/föö-bar", base_url="http://☃.net/")
+    assert b.path == "/f%C3%B6%C3%B6-bar"
+    assert b.base_url == "http://xn--n3h.net/"
 
 
 @pytest.mark.parametrize("buffered", (True, False))
@@ -618,7 +661,7 @@ def test_run_wsgi_apps(buffered, iterable):
         leaked_data.append("harhar")
         start_response("200 OK", [("Content-Type", "text/html")])
 
-        class Rv(object):
+        class Rv:
             def __iter__(self):
                 yield "Hello "
                 yield "World"
@@ -633,9 +676,9 @@ def test_run_wsgi_apps(buffered, iterable):
         if iterable:
             app = iterable_middleware(app)
         app_iter, status, headers = run_wsgi_app(app, {}, buffered=buffered)
-        strict_eq(status, "200 OK")
-        strict_eq(list(headers), [("Content-Type", "text/html")])
-        strict_eq("".join(app_iter), "Hello World!")
+        assert status == "200 OK"
+        assert list(headers) == [("Content-Type", "text/html")]
+        assert "".join(app_iter) == "Hello World!"
 
         if hasattr(app_iter, "close"):
             app_iter.close()
@@ -645,7 +688,6 @@ def test_run_wsgi_apps(buffered, iterable):
 @pytest.mark.parametrize("buffered", (True, False))
 @pytest.mark.parametrize("iterable", (True, False))
 def test_lazy_start_response_empty_response_app(buffered, iterable):
-    @implements_iterator
     class app:
         def __init__(self, environ, start_response):
             self.start_response = start_response
@@ -660,16 +702,15 @@ def test_lazy_start_response_empty_response_app(buffered, iterable):
     if iterable:
         app = iterable_middleware(app)
     app_iter, status, headers = run_wsgi_app(app, {}, buffered=buffered)
-    strict_eq(status, "200 OK")
-    strict_eq(list(headers), [("Content-Type", "text/html")])
-    strict_eq("".join(app_iter), "")
+    assert status == "200 OK"
+    assert list(headers) == [("Content-Type", "text/html")]
+    assert "".join(app_iter) == ""
 
 
 def test_run_wsgi_app_closing_iterator():
     got_close = []
 
-    @implements_iterator
-    class CloseIter(object):
+    class CloseIter:
         def __init__(self):
             self.iterated = False
 
@@ -707,7 +748,7 @@ def iterable_middleware(app):
     def inner(environ, start_response):
         rv = app(environ, start_response)
 
-        class Iterable(object):
+        class Iterable:
             def __iter__(self):
                 return iter(rv)
 
@@ -725,17 +766,15 @@ def test_multiple_cookies():
     @Request.application
     def test_app(request):
         response = Response(repr(sorted(request.cookies.items())))
-        response.set_cookie(u"test1", b"foo")
-        response.set_cookie(u"test2", b"bar")
+        response.set_cookie("test1", b"foo")
+        response.set_cookie("test2", b"bar")
         return response
 
-    client = Client(test_app, Response)
+    client = Client(test_app)
     resp = client.get("/")
-    strict_eq(resp.data, b"[]")
+    assert resp.data == b"[]"
     resp = client.get("/")
-    strict_eq(
-        resp.data, to_bytes(repr([("test1", u"foo"), ("test2", u"bar")]), "ascii")
-    )
+    assert resp.data == _to_bytes(repr([("test1", "foo"), ("test2", "bar")]), "ascii")
 
 
 def test_correct_open_invocation_on_redirect():
@@ -753,15 +792,15 @@ def test_correct_open_invocation_on_redirect():
         return Response(str(request.environ["werkzeug._foo"]))
 
     c = MyClient(test_app, response_wrapper=Response)
-    strict_eq(c.get("/").data, b"1")
-    strict_eq(c.get("/").data, b"2")
-    strict_eq(c.get("/").data, b"3")
+    assert c.get("/").data == b"1"
+    assert c.get("/").data == b"2"
+    assert c.get("/").data == b"3"
 
 
 def test_correct_encoding():
-    req = Request.from_values(u"/\N{SNOWMAN}", u"http://example.com/foo")
-    strict_eq(req.script_root, u"/foo")
-    strict_eq(req.path, u"/\N{SNOWMAN}")
+    req = Request.from_values("/\N{SNOWMAN}", "http://example.com/foo")
+    assert req.script_root == "/foo"
+    assert req.path == "/\N{SNOWMAN}"
 
 
 def test_full_url_requests_with_args():
@@ -771,11 +810,11 @@ def test_full_url_requests_with_args():
     def test_app(request):
         return Response(request.args["x"])
 
-    client = Client(test_app, Response)
+    client = Client(test_app)
     resp = client.get("/?x=42", base)
-    strict_eq(resp.data, b"42")
+    assert resp.data == b"42"
     resp = client.get("http://www.example.com/?x=23", base)
-    strict_eq(resp.data, b"23")
+    assert resp.data == b"23"
 
 
 def test_delete_requests_with_form():
@@ -783,21 +822,21 @@ def test_delete_requests_with_form():
     def test_app(request):
         return Response(request.form.get("x", None))
 
-    client = Client(test_app, Response)
+    client = Client(test_app)
     resp = client.delete("/", data={"x": 42})
-    strict_eq(resp.data, b"42")
+    assert resp.data == b"42"
 
 
 def test_post_with_file_descriptor(tmpdir):
-    c = Client(Response(), response_wrapper=Response)
+    c = Client(Response())
     f = tmpdir.join("some-file.txt")
     f.write("foo")
-    with open(f.strpath, mode="rt") as data:
+    with open(f.strpath) as data:
         resp = c.post("/", data=data)
-    strict_eq(resp.status_code, 200)
+    assert resp.status_code == 200
     with open(f.strpath, mode="rb") as data:
         resp = c.post("/", data=data)
-    strict_eq(resp.status_code, 200)
+    assert resp.status_code == 200
 
 
 def test_content_type():
@@ -805,13 +844,13 @@ def test_content_type():
     def test_app(request):
         return Response(request.content_type)
 
-    client = Client(test_app, Response)
+    client = Client(test_app)
 
     resp = client.get("/", data=b"testing", mimetype="text/css")
-    strict_eq(resp.data, b"text/css; charset=utf-8")
+    assert resp.data == b"text/css; charset=utf-8"
 
     resp = client.get("/", data=b"testing", mimetype="application/octet-stream")
-    strict_eq(resp.data, b"application/octet-stream")
+    assert resp.data == b"application/octet-stream"
 
 
 def test_raw_request_uri():
@@ -821,7 +860,41 @@ def test_raw_request_uri():
         request_uri = request.environ["REQUEST_URI"]
         return Response("\n".join((path_info, request_uri)))
 
-    client = Client(app, Response)
+    client = Client(app)
     response = client.get("/hello%2fworld")
     data = response.get_data(as_text=True)
     assert data == "/hello/world\n/hello%2fworld"
+
+    response = client.get("/?a=b")
+    assert response.get_data(as_text=True) == "/\n/?a=b"
+
+    response = client.get("/%3f?")  # escaped ? in path, and empty query string
+    assert response.get_data(as_text=True) == "/?\n/%3f?"
+
+
+def test_raw_request_uri_redirect():
+    @Request.application
+    def app(request):
+        if request.path == "/first":
+            return redirect("/second")
+
+        path_info = request.path
+        request_uri = request.environ["REQUEST_URI"]
+        return Response("\n".join((path_info, request_uri)))
+
+    client = Client(app)
+    response = client.get("/first", follow_redirects=True)
+    data = response.get_data(as_text=True)
+    assert data == "/second\n/second"
+
+
+def test_deprecated_tuple():
+    app = Request.application(lambda r: Response())
+    client = Client(app)
+    response = client.get()
+
+    with pytest.deprecated_call():
+        assert len(list(response)) == 3
+
+    with pytest.deprecated_call():
+        assert response[1] == "200 OK"
